----------------------------------------------------------------------------------
-- Company:             www.kampis-elektroecke.de
-- Engineer:            Daniel Kampert
-- 
-- Create Date:         10.03.2015 09:26:02
-- Design Name: 
-- Module Name:         Display_RAM - Display_RAM_Arch
-- Project Name: 
-- Target Devices:      XC7Z010CLG400-1
-- Tool Versions:       Vivado 2014.4
-- Description:         Display-RAM for VGA-Core 
--                      (640x480, Charsize 8x8 -> 80x60 Chars per Screen)
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Display_RAM is
    Generic (
            Addr_Width : integer := 13;
            Data_Width : integer := 8
            );
    Port (  Clock : in STD_LOGIC;
            Read_Write : in STD_LOGIC;                                                          -- Schreib-/Leseumschaltung, 1 - Lesen, 0 - Schreiben
            Address : in STD_LOGIC_VECTOR((Addr_Width - 1) downto 0);                           -- Adresse der Zeichen auf dem Bildschirm
            Data_Out : out STD_LOGIC_VECTOR ((Data_Width - 1) downto 0);                        -- Aktuelles Zeichen auf dem Bildschirm
            Data_In : in STD_LOGIC_VECTOR ((Data_Width - 1) downto 0)                           -- Zeichen, welches auf dem Bildschirm ausgegeben werden soll
            );
end Display_RAM;

architecture Display_RAM_Arch of Display_RAM is

    type Display_RAM_Type is array (0 to (2**Address'length) - 1) of std_logic_vector(Data_In'range);
    signal Read_Address : std_logic_vector(Address'range);
    signal RAM : Display_RAM_Type := (
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    x"48",
    x"61",
    x"6c",
    x"6c",
    x"6f",
    
--    x"00",
--    x"57",
--    x"65",
--    x"6c",
--    x"74",
    others=> "00000000"
    );
    
begin
    
    process(Clock)
    begin
        if(rising_edge(Clock)) then
            if(Read_Write = '0') then
                RAM(to_integer(unsigned(Read_Address))) <= Data_In;
            end if;
            
            Read_Address <= Address;
            
        end if;
    end process;

    Data_Out <= RAM(to_integer(unsigned(Read_Address)));   

end Display_RAM_Arch;